@@ -28,18 +28,27 @@
 
 #include "eog-error-message-area.h"
 #include "eog-image.h"
-#include "eog-util.h"
 
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <glib-object.h>
 #include <gtk/gtk.h>
+#include <gio/gdesktopappinfo.h>
+
+/* gboolean <-> gpointer conversion macros taken from gedit */
+#ifndef GBOOLEAN_TO_POINTER
+#define GBOOLEAN_TO_POINTER(i) (GINT_TO_POINTER ((i) ? 2 : 1))
+#endif
+#ifndef GPOINTER_TO_BOOLEAN
+#define GPOINTER_TO_BOOLEAN(i) ((gboolean) ((GPOINTER_TO_INT (i) == 2) ? TRUE : FALSE))
+#endif
 
 typedef enum {
 	EOG_ERROR_MESSAGE_AREA_NO_BUTTONS    = 0,
 	EOG_ERROR_MESSAGE_AREA_CANCEL_BUTTON = 1 << 0,
 	EOG_ERROR_MESSAGE_AREA_RELOAD_BUTTON = 1 << 1,
-	EOG_ERROR_MESSAGE_AREA_SAVEAS_BUTTON = 1 << 2
+	EOG_ERROR_MESSAGE_AREA_SAVEAS_BUTTON = 1 << 2,
+	EOG_ERROR_MESSAGE_AREA_OPEN_WITH_EVINCE_BUTTON = 1 << 3
 } EogErrorMessageAreaButtons;
 
 static void
@@ -104,17 +113,10 @@ set_message_area_text_and_icon (GtkInfoBar   *message_area,
 	gtk_box_pack_start (GTK_BOX (gtk_info_bar_get_content_area (GTK_INFO_BAR (message_area))), hbox_content, TRUE, TRUE, 0);
 }
 
-static GtkWidget *
-create_error_message_area (const gchar                 *primary_text,
-			   const gchar                 *secondary_text,
-			   EogErrorMessageAreaButtons   buttons)
+static void
+add_message_area_buttons (GtkWidget *message_area,
+			  EogErrorMessageAreaButtons buttons)
 {
-	GtkWidget *message_area;
-
-	/* create a new message area */
-	message_area = gtk_info_bar_new ();
-
-	/* add requested buttons to the message area */
 	if (buttons & EOG_ERROR_MESSAGE_AREA_CANCEL_BUTTON)
 		gtk_info_bar_add_button (GTK_INFO_BAR (message_area),
 					 _("_Cancel"),
@@ -129,6 +131,24 @@ create_error_message_area (const gchar                 *primary_text,
 		gtk_info_bar_add_button (GTK_INFO_BAR (message_area),
 					 _("Save _As…"),
 					 EOG_ERROR_MESSAGE_AREA_RESPONSE_SAVEAS);
+	if (buttons & EOG_ERROR_MESSAGE_AREA_OPEN_WITH_EVINCE_BUTTON)
+		gtk_info_bar_add_button (GTK_INFO_BAR (message_area),
+					 _("Open with Document Viewer"),
+					 EOG_ERROR_MESSAGE_AREA_RESPONSE_OPEN_WITH_EVINCE);
+}
+
+static GtkWidget *
+create_error_message_area (const gchar                 *primary_text,
+			   const gchar                 *secondary_text,
+			   EogErrorMessageAreaButtons   buttons)
+{
+	GtkWidget *message_area;
+
+	/* create a new message area */
+	message_area = gtk_info_bar_new ();
+
+	/* add requested buttons to the message area */
+	add_message_area_buttons (message_area, buttons);
 
 	/* set message type */
 	gtk_info_bar_set_message_type (GTK_INFO_BAR (message_area),
@@ -143,6 +163,32 @@ create_error_message_area (const gchar                 *primary_text,
 	return message_area;
 }
 
+static GtkWidget *
+create_info_message_area (const gchar                 *primary_text,
+			  const gchar                 *secondary_text,
+			  EogErrorMessageAreaButtons   buttons)
+{
+	GtkWidget *message_area;
+
+	/* create a new message area */
+	message_area = gtk_info_bar_new ();
+
+	/* add requested buttons to the message area */
+	add_message_area_buttons (message_area, buttons);
+
+	/* set message type */
+	gtk_info_bar_set_message_type (GTK_INFO_BAR (message_area),
+				       GTK_MESSAGE_INFO);
+
+	/* set text and icon */
+	set_message_area_text_and_icon (GTK_INFO_BAR (message_area),
+					"dialog-information",
+					primary_text,
+					secondary_text);
+
+	return message_area;
+}
+
 /**
  * eog_image_load_error_message_area_new: (skip):
  * @caption:
@@ -172,7 +218,7 @@ eog_image_load_error_message_area_new (const gchar  *caption,
 	error_message = g_strdup_printf (_("Could not load image '%s'."),
 					 pango_escaped_caption);
 
-	message_details = eog_util_make_valid_utf8 (error->message);
+	message_details = g_strdup (error->message);
 
 	message_area = create_error_message_area (error_message,
 						  message_details,
@@ -214,7 +260,7 @@ eog_image_save_error_message_area_new (const gchar  *caption,
 	error_message = g_strdup_printf (_("Could not save image '%s'."),
 					 pango_escaped_caption);
 
-	message_details = eog_util_make_valid_utf8 (error->message);
+	message_details = g_strdup (error->message);
 
 	message_area = create_error_message_area (error_message,
 						  message_details,
@@ -272,3 +318,49 @@ eog_no_images_error_message_area_new (GFile *file)
 
 	return message_area;
 }
+
+static gpointer
+_check_evince_availability(gpointer data)
+{
+	gboolean result = FALSE;
+	GDesktopAppInfo *app_info;
+
+	app_info = g_desktop_app_info_new ("evince.desktop");
+	if (app_info) {
+		result = TRUE;
+		g_object_unref (app_info);
+	}
+	return GBOOLEAN_TO_POINTER(result);
+}
+
+GtkWidget *
+eog_multipage_error_message_area_new(void)
+{
+	static GOnce evince_is_available = G_ONCE_INIT;
+	EogErrorMessageAreaButtons buttons = EOG_ERROR_MESSAGE_AREA_NO_BUTTONS;
+	GtkWidget *message_area;
+	const gchar *info_message;
+
+	g_once (&evince_is_available, _check_evince_availability, NULL);
+
+	if (GPOINTER_TO_BOOLEAN (evince_is_available.retval))
+	{
+		buttons = EOG_ERROR_MESSAGE_AREA_OPEN_WITH_EVINCE_BUTTON;
+		info_message = N_("This image contains multiple pages. "
+				  "Image Viewer displays only the first page.\n"
+				  "Do you want to open the image with the Document Viewer to see all pages?");
+	} else {
+		buttons = EOG_ERROR_MESSAGE_AREA_NO_BUTTONS;
+		info_message = N_("This image contains multiple pages. "
+				  "Image Viewer displays only the first page.\n"
+				  "You may want to install the Document Viewer to see all pages.");
+	}
+
+	message_area = create_info_message_area (gettext (info_message),
+						 NULL,
+						 buttons);
+	gtk_info_bar_set_show_close_button (GTK_INFO_BAR (message_area),
+					    TRUE);
+
+	return message_area;
+}
